package people;


public class TestEmployee extends AbstractTest {

	public static void main(String[] args) throws InstantiationException, IllegalAccessException {
		new TestEmployee();
	}

	protected void runTests() {
		this.testExpected = 24;
		try {
			testSuperTypes();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetAndSetSalary();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetUniqueId();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testToString();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testCopyConstructor();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testEquals();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		// we might test a EmployeeImpl by running TestPersonImpl
	}

	public void testSuperTypes() {
		EmployeeImpl p = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);
		assertInstanceOf(p, people.Employee.class);
		assertInstanceOf(p, people.Person.class);
		assertInstanceOf(p, people.PersonImpl.class);
	}

	public void testGetAndSetSalary() {
		EmployeeImpl p = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);

		double expected = 2501.;
		double value = p.getSalary();
		assertEquals(expected, value);

		p.setSalary(3666);
		expected = 3666;
		value = p.getSalary();
		assertEquals(expected, value);
	}

	public void testGetUniqueId() {
		EmployeeImpl p = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);
		assertEquals("N2501UM3N", p.getUniqueId());
	}

	public void testToString() {
		EmployeeImpl p = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);

		String expectedS = "[N2501UM3N] Motoko Kusanagi (age: 21) salary: 2501.0 euros";
		String valueS = p.toString();
		assertEquals(expectedS, valueS);
	}

	public void testCopyConstructor() {
		EmployeeImpl p = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);
		EmployeeImpl copy = new EmployeeImpl(p);

		assertTrue(p != copy, "references should not be equal");
		assertEquals(p.getName(), copy.getName());
		assertEquals(p.getAge(), copy.getAge());
		assertEquals(p.getSalary(), copy.getSalary());
		assertEquals(p.getUniqueId(), copy.getUniqueId());
	}

	public void testEquals() {
		Employee p = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);
		Employee p2 = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);
		Employee pDifferentName = new EmployeeImpl("N2501UM3N", "Puppet Master", 21, 2501.);
		Employee pDifferentAge = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 25, 2501.);
		Employee pDifferentId = new EmployeeImpl("NXXXXUM3N", "Motoko Kusanagi", 21, 2501.);
		Employee pDifferentSalary = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2509.);
		Employee pNullId = new EmployeeImpl(null, "Motoko Kusanagi", 21, 2501.);
		Employee pNullName = new EmployeeImpl("N2501UM3N", null, 21, 2501.);

		assertTrue(!p.equals(new Object()), "objects should not be equal");
		assertTrue(p.equals(p), "object should be equal to itself");
		assertTrue(p.equals(p2), "objects should be equal");
		assertTrue(!p.equals(pDifferentName), "objects should not be equal");
		assertTrue(!p.equals(pDifferentAge), "objects should not be equal");
		assertTrue(!p.equals(pDifferentId), "objects should not be equal");
		assertTrue(!p.equals(pDifferentSalary), "objects should not be equal");
		assertTrue(!p.equals(null), "object should not be equal to null reference");
		assertTrue(!p.equals(pNullId), "objects should not be equal");
		assertTrue(!p.equals(pNullName), "objects should not be equal");
		assertTrue(!pNullId.equals(p), "objects should not be equal");
		assertTrue(!pNullName.equals(p), "objects should not be equal");
	}

}
